# Duden Korrektur TinyMCE Plugin

This is a TinyMCE Plugin for using DKS (Duden Korrektur Server) with TinyMCE. It is compatible with TinyMCE 3, TinyMCE 4, and TinyMCE 5.

### Requirements

- A functioning TinyMCE implementation.
- Access to the server to edit configuration and uploading files.
- Server with PHP, if you want to use the PHP Proxy Sript (which is used by Default)
- Server with PERL, if you want to use the PERL Proxy Script

### Installation Instructions

1. Unzip/Copy the "duden" directory into the "plugins" directory of your TinyMCE Installation. The path should be "plugins/duden".

2. If you want to connect to the DKS-Server through a Proxy-Script open the associated file. We have an implemenation in [PERL](https://www.perl.org/) and [PHP](https://www.php.net/). They are located in the **'perl'**/**'php'** subdirectory of the Duden TinyMCE Plugin. Insert your credentials into these files. Do **_not_** provide any credentials if the DKS is not protected by username and password. You can skip this step, if you want to use an API-User to make requests to the DKS. By default, the Duden Plugin tries to use the PHP Script.

3. Verify that the DKS is reachable. If the api is not secured and publicly available you can try by using the command "wget $DCS_URL/version"

4. Edit your TinyMCE configuration. Add "duden" to the list of plugins.

5. Add "duden_check" as a button some where on your toolbar

6. To enable the advanced language selection mode, where the language can be set for individual paragraphs, add "duden_lang" to the toolbar setting. (This is only supported in TinyMCE 3 at the moment).

### Initialization Examples

**TinyMCE 3:**
```javascript
tinymce.init({
    selector: "textarea",
    plugins: "duden",
    paste_as_text: true,
    theme : "advanced",
    theme_advanced_buttons1: "duden_menu,duden_check,duden_lang"
});
```

**TinyMCE 4 / TinyMCE 5:**
```javascript
tinymce.init({
    selector: "textarea",
    plugins: "duden",
    toolbar: 'duden_check'
});
```

### Plugin Options

You can set various options via the "duden_settings" object. Here is a short example code for TinyMCE 5 with all available settings with explanaitions in the comment sections. You can easily adapt it to TinyMCE 4 or TinyMCE 3. All the settings work the same in the differen TinyMCE Versions. Also, each setting is optional, so if omitted it uses the default setting:

```javascript
tinymce.init({
    selector: "textarea",
    plugins: "duden",
    toolbar: 'duden_check',
    duden_settings: {
        // If apiEndpoint is ommited it will default to the PHP Proxy Script,
        // so make sure that you have updated the script with your credentials
        // If you set an API Endpoint directly, please make sure that the DKS
        // is available from your network or provide separate credentials
        apiEndpoint: 'http://127.0.0.1:8080/api',

        // If the API is not publicly available please add the
        // credentials to an API user
        // You can either use a username/password combination or an apiKey
        apiCredentials: {
            username: "duden",
            password: "duden" /*,
            apiKey: "??????????" */
        },

        // Which property sets should the DKS use
        propertySets: ["base"],

        // Which dictionaries sets should the DKS use
        dictionaries: [],

        // Can be used to activate debug output to browser console.
        debug: false,

        // If a user manually adds hyphenation, DKCE will auto detect it and
        // replaces the '-' char with a soft hyphen
        detectManualHyphenation: true,

        // Which language should the DKCE UI use
        // Defaults to the browser language of the user
        language: "de",

        // Which fallback Language should the DKCE Plugin use, if the
        // main language cannot be found. This is especially important when
        // you ommit the language setting, and the language is detected
        // automatically by the users browser settings.
        fallbackLanguage: "de",

        // Should the plugin be activated. If set to false, you can activate
        // it at a later point
        activated: true,

        // Optional: Defines when the Spellchecking Pop Up appears,
        // following options are possible:
        //      'mouseOver'  - Automatically opens pop up when you hover an error highlight (Default)
        //      'leftClick'  - Opens pop up when you left click on an error highlight
        //      'rightClick' - Opens pop up when you right click on an error highlight
        openContextMenuOn: 'mouseOver',

        // Can be used to disable the Duden-Status-Panel at the top of the screen
        // (which indicates network communication)
        // Defaults to: false
        hideStatusPanel: false,

        // You can set the options how the DKS should behave, but keep in mind
        // that the DKS Server may overwrite these settings
        // Please check the DKS Readme if you want to learn more about them.
        hyphenation: true,
        hyphenationStandard: "combined",
        hyphenationInStem: true,
        hyphenationUnaest: false,

        // When set it will ignore spellchecks for all child elements who has
        // the given attribute in the HTML set (can be string or array of strings)
        ignoreAttributes: [],

        // When set it will separate spellchecks for all child elements who has
        // the given attribute in the HTML set (can be string or array of
        // strings). This can be useful for footnotes in HTML code, for example.
        layerAttributes: [],

        // Use single word mode for layers defined in layer Attributes (except when it detects
        // end of sentence characters. Default is true.)
        useSingleWordModeOnLayers: true,

        // When set it will ignore spellchecks for all child elements who has
        // the given css class in the HTML set (can be string or array of strings)
        ignoreClasses: [],

        // Every single character of this string will be ignored during the
        // correction check. This can be useful, for example, if the editor
        // inserts characters for marking that are to be ignored by the spell checker.
        ignoreCharacters: "",

        // When set it will ignore spellchecks for paragraphs that have a
        // language attribute attached that are not defined here. If ommited all languages will be checked.
        checkLanguages: [],

        correctionProposals: true,
        singleWordMode: false,
        styleForeign: false,
        styleOld: false,
        styleRegional: false,
        styleColloquial: false,
        styleFiller: false,
        markupMode: "text",
        messageLanguage: "de-DE",
        textLanguage: "de-DE",
        messageLevel: 3,
        spellcheckingLevel: 1,
        orthographyStandard: "duden",

        // Sets the dictonary where proposals will be submitted to. (Default is 'Proposals')
        proposalDictionary: "Proposals",

        // Use markup mode instead of overlay mode (which is the default)
        overlayMode: false,

        // If set to true (and instance is in MarkUp Mode) all markup code is
        // removed from selections (and readded when selection is removed)
        removeMarkupForSelection: false,

        // colors can be used to define the appearance of the highlights. Please keep in mind
        // that in markup mode, the colors may be overwritten by included css Files
        colors: {
            spellingError: '#faa6ae',
            gramarError: '#abe89e',
            rejectedWord: '#abe89e',
            styleError: '#ffd7a8',
            glossaryInfo: '#ffff00'
        },

        // Array of CSS Files to use for DKCE. Please keep in mind that most
        // of the css classes only make sense in markup mode
        cssFiles: ['assets/css/dkce.css'],

        // Path to the dkce javascript source files. Is important so the
        // plugin finds the assets like images and localization files.
        path: "dkce",

        // If a canvas exceeds this size (in pixels) in overlay mode,
        // it is split into several individual ones
        maxCanvasWidth: 512,
        maxCanvasHeight: 512,

        // Makes the overlay canvas elements visible (for debugging purposes)
        debugCanvas: false
    }
});
```
#### Using a configuration file

Optionally, you can use a configuration file to specify options instead of (or in addition to) the options provided in the call to `tinymce.init()`. To use this mechanism, create a file named 'duden-config.json' in the "duden" directory and provide the the desired options and their values in JSON notation.

For example:
```json
{
  "propertySets": ["set1","set2"],
  "textLanguage": "de-CH"
}
```
The plugin ships without a configuration file to avoid overwriting of user defined settings when updating the plugin. The presence of a configuration file is not required for the proper operation of the plugin, and corresponding "404 File not found" errors can be safely ignored.

### Upgrading from previous versions
If you have created a configuration file, please make a backup copy of this file before upgrading, e. g. by copying it to a temporary directory.

If you are upgrading from a previous version, please make sure that you completely delete the "duden" folder within your TinyMCE installation and do not forget to edit the proxy scripts again. Do not keep your old scripts as there might be internal changes that would lead to incompatibilities. With older versions of our TinyMCE plugin, you may have used different settings to initialise the TinyMCE Duden plugin. These are still working, but if possible, use the new "duden_settings" object, as older settings may become obsolete in the future.

Please check especially the correct path in the Proxy Script (if you use one). In foreign versions the path need to include the '/api' slug in the host configuration setting, while in newer version this have to be ommited.

If you are using an additional configuration file, please copy file back to the  'duden' directory and verify that the content of this file is compatible with the options suppported by the current version of the plugin.

## License

(C) Copyright 2022, 2023 by EPC Consulting und Software GmbH.
